@@ -13,16 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.api.v2 import attributes as attr
 from neutron.common import rpc as n_rpc
 from neutron import context as neutron_context
 from neutron import manager
 from neutron.plugins.common import constants as n_const
+from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron_fwaas._i18n import _LW
+from neutron_fwaas._i18n import _LI, _LW
 from neutron_fwaas.common import fwaas_constants as f_const
 from neutron_fwaas.db.firewall import firewall_db
 from neutron_fwaas.db.firewall import firewall_router_insertion_db
@@ -60,18 +60,22 @@ class FirewallCallbacks(object):
     def firewall_deleted(self, context, firewall_id, **kwargs):
         """Agent uses this to indicate firewall is deleted."""
         LOG.debug("firewall_deleted() called")
-        with context.session.begin(subtransactions=True):
-            fw_db = self.plugin._get_firewall(context, firewall_id)
-            # allow to delete firewalls in ERROR state
-            if fw_db.status in (n_const.PENDING_DELETE, n_const.ERROR):
-                self.plugin.delete_db_firewall_object(context, firewall_id)
-                return True
-            else:
-                LOG.warning(_LW('Firewall %(fw)s unexpectedly deleted by '
-                                'agent, status was %(status)s'),
-                            {'fw': firewall_id, 'status': fw_db.status})
-                fw_db.update({"status": n_const.ERROR})
-                return False
+        try:
+            with context.session.begin(subtransactions=True):
+                fw_db = self.plugin._get_firewall(context, firewall_id)
+                # allow to delete firewalls in ERROR state
+                if fw_db.status in (n_const.PENDING_DELETE, n_const.ERROR):
+                    self.plugin.delete_db_firewall_object(context, firewall_id)
+                    return True
+                else:
+                    LOG.warning(_LW('Firewall %(fw)s unexpectedly deleted by '
+                                    'agent, status was %(status)s'),
+                                {'fw': firewall_id, 'status': fw_db.status})
+                    fw_db.update({"status": n_const.ERROR})
+                    return False
+        except fw_ext.FirewallNotFound:
+            LOG.info(_LI('Firewall %s already deleted'), firewall_id)
+            return True
 
     def get_firewalls_for_tenant(self, context, **kwargs):
         """Agent uses this to get all firewalls and rules for a tenant."""
@@ -148,7 +152,7 @@ class FirewallPlugin(
         self.start_rpc_listeners()
 
         self.agent_rpc = FirewallAgentApi(
-            f_const.L3_AGENT,
+            f_const.FW_AGENT,
             cfg.CONF.host
         )
         firewall_db.subscribe()
@@ -205,7 +209,7 @@ class FirewallPlugin(
         # pop router_id as this goes in the router association db
         # and not firewall db
         router_ids = firewall['firewall'].pop('router_ids', None)
-        if router_ids == attr.ATTR_NOT_SPECIFIED:
+        if router_ids == constants.ATTR_NOT_SPECIFIED:
             # old semantics router-ids keyword not specified pick up
             # all routers on tenant.
             l3_plugin = manager.NeutronManager.get_service_plugins().get(
